@@ -1,5 +1,6 @@
 """Core filesystem functionality and utilities."""
 
+import inspect
 import base64
 import os
 import posixpath
@@ -11,14 +12,17 @@ from typing import Optional, Union
 import fsspec
 import requests
 from fsspec import filesystem as fsspec_filesystem
+from fsspec.core import split_protocol
 from fsspec.implementations.cache_mapper import AbstractCacheMapper
 from fsspec.implementations.cached import SimpleCacheFileSystem
 from fsspec.implementations.dirfs import DirFileSystem
 from fsspec.implementations.memory import MemoryFile
+from fsspec.registry import known_implementations
 
 from ..storage_options.base import BaseStorageOptions
 from ..storage_options.core import from_dict as storage_options_from_dict
 from ..utils.logging import get_logger
+
 # from fsspec.utils import infer_storage_options
 from .ext import AbstractFileSystem
 
@@ -70,43 +74,21 @@ class FileNameCacheMapper(AbstractCacheMapper):
             >>> print(mapper("data/nested/file.txt"))
             'data/nested/file.txt'
         """
+        # os.makedirs(
+        #     posixpath.dirname(posixpath.join(self.directory, path)), exist_ok=True
+        # )
         os.makedirs(
             posixpath.dirname(posixpath.join(self.directory, path)), exist_ok=True
         )
+
         return path
 
 
 class MonitoredSimpleCacheFileSystem(SimpleCacheFileSystem):
-    """Enhanced caching filesystem with monitoring and improved path handling.
-
-    This filesystem extends SimpleCacheFileSystem to provide:
-    - Verbose logging of cache operations
-    - Improved path mapping for cache files
-    - Enhanced synchronization capabilities
-    - Better handling of parallel operations
-
-    Attributes:
-        _verbose (bool): Whether to print verbose cache operations
-        _mapper (FileNameCacheMapper): Maps remote paths to cache paths
-        storage (list[str]): List of cache storage locations
-        fs (AbstractFileSystem): Underlying filesystem being cached
-
-    Example:
-        >>> from fsspec import filesystem
-        >>> s3_fs = filesystem("s3")
-        >>> cached_fs = MonitoredSimpleCacheFileSystem(
-        ...     fs=s3_fs,
-        ...     cache_storage="/tmp/cache",
-        ...     verbose=True
-        ... )
-        >>> # Use cached_fs like any other filesystem
-        >>> files = cached_fs.ls("my-bucket/")
-    """
-
     def __init__(
         self,
         fs: Optional[fsspec.AbstractFileSystem] = None,
-        cache_storage: Union[str, list[str]] = "~/.cache/fsspec",
+        cache_storage: str = "~/.cache/fsspec",
         verbose: bool = False,
         **kwargs,
     ):
@@ -129,58 +111,183 @@ class MonitoredSimpleCacheFileSystem(SimpleCacheFileSystem):
         """
         self._verbose = verbose
 
-        # Handle cache storage configuration
-        if isinstance(cache_storage, str):
-            cache_storage = [cache_storage]
-
-        self.storage = cache_storage
-
-        # Set up cache mapper for preserving directory structure
-        if len(cache_storage) == 1:
-            self._mapper = FileNameCacheMapper(cache_storage[0])
-            kwargs["cache_mapper"] = self._mapper
-
-        # Initialize with expanded cache storage paths
-        expanded_storage = [os.path.expanduser(path) for path in cache_storage]
-        super().__init__(fs=fs, cache_storage=expanded_storage, **kwargs)
+        # # Initialize with expanded cache storage paths
+        # expanded_storage = os.path.expanduser(cache_storage)
+        # super().__init__(
+        #     fs=fs,
+        #     cache_storage=expanded_storage,
+        #     cache_mapper=FileNameCacheMapper(expanded_storage),
+        #     **kwargs,
+        # )
+        # kwargs["cache_storage"] = os.path.join(kwargs.get("cache_storage"), "123")
+        super().__init__(fs=fs, cache_storage=cache_storage, **kwargs)
+        self._mapper = FileNameCacheMapper(cache_storage)
 
         if self._verbose:
-            logger.info(
-                f"Initialized cache filesystem with storage: {expanded_storage}"
-            )
+            logger.info(f"Initialized cache filesystem with storage: {cache_storage}")
 
-    def _check_cache(self, path: str) -> Optional[str]:
-        """Check if file exists in cache and return cache path if found.
-
-        Args:
-            path: Remote file path to check
-
-        Returns:
-            Cache file path if found, None otherwise
+    def open(self, path, mode="rb", **kwargs):
         """
-        result = super()._check_cache(path)
-        if self._verbose and result:
-            logger.info(f"Cache hit for {path} -> {result}")
-        return result
-
-    def _check_file(self, path: str) -> str:
-        """Ensure file is in cache, downloading if necessary.
-
-        Args:
-            path: Remote file path
-
-        Returns:
-            Local cache path for the file
+        Open a file. If the file's path does not match the cache regex, bypass the
+        caching and read directly from the underlying filesystem.
         """
+        # if not ICEBERG_FILE_REGEX.search(path):
+        #     # bypass caching.
+        #     return self.fs.open(path, mode=mode, **kwargs)
+
+        return super().open(path, mode=mode, **kwargs)
+
+    def _check_file(self, path):
+        # self._check_cache()
         if self._verbose:
             logger.info(f"Checking file: {path}")
+        # cache_path = self._mapper(path)
 
-        result = super()._check_file(path)
+        for storage in self.storage:
+            fn = os.path.join(storage, path)
+            if os.path.exists(fn):
+                return fn
+            # else:
+            #    self.open(path, mode="rb").close()
+            #    logger.info(f"Downloading {self.protocol[0]}://{path}")
 
-        if self._verbose:
-            logger.info(f"File available at: {result}")
+    def size(self, path: str) -> int:
+        """Get size of file in bytes.
 
-        return result
+        Checks cache first, falls back to remote filesystem.
+
+        Args:
+            path: Path to file
+
+        Returns:
+            int: Size of file in bytes
+
+        Example:
+            >>> fs = MonitoredSimpleCacheFileSystem(
+            ...     fs=remote_fs,
+            ...     cache_storage="/tmp/cache"
+            ... )
+            >>> size = fs.size("large_file.dat")
+            >>> print(f"File size: {size} bytes")
+        """
+        cached_file = self._check_file(self._strip_protocol(path))
+        if cached_file is None:
+            return self.fs.size(path)
+        else:
+            return posixpath.getsize(cached_file)
+
+    def sync_cache(self, reload: bool = False) -> None:
+        """Synchronize cache with remote filesystem.
+
+        Downloads all files in remote path to cache if not present.
+
+        Args:
+            reload: Whether to force reload all files, ignoring existing cache
+
+        Example:
+            >>> fs = MonitoredSimpleCacheFileSystem(
+            ...     fs=remote_fs,
+            ...     cache_storage="/tmp/cache"
+            ... )
+            >>> # Initial sync
+            >>> fs.sync_cache()
+            >>>
+            >>> # Force reload all files
+            >>> fs.sync_cache(reload=True)
+        """
+        if reload:
+            if hasattr(self, "clear_cache"):
+                self.clear_cache()
+
+        files = self.glob("**/*")
+        [self.open(f, mode="rb").close() for f in files if self.isfile(f)]
+
+    def __getattribute__(self, item):
+        if item in {
+            # new items
+            "size",
+            "glob",
+            # previous
+            "load_cache",
+            "_open",
+            "save_cache",
+            "close_and_update",
+            "sync_cache",
+            "__init__",
+            "__getattribute__",
+            "__reduce__",
+            "_make_local_details",
+            "open",
+            "cat",
+            "cat_file",
+            "cat_ranges",
+            "get",
+            "read_block",
+            "tail",
+            "head",
+            "info",
+            "ls",
+            "exists",
+            "isfile",
+            "isdir",
+            "_check_file",
+            "_check_cache",
+            "_mkcache",
+            "clear_cache",
+            "clear_expired_cache",
+            "pop_from_cache",
+            "local_file",
+            "_paths_from_path",
+            "get_mapper",
+            "open_many",
+            "commit_many",
+            "hash_name",
+            "__hash__",
+            "__eq__",
+            "to_json",
+            "to_dict",
+            "cache_size",
+            "pipe_file",
+            "pipe",
+            "start_transaction",
+            "end_transaction",
+            "sync_cache",
+        }:
+            # all the methods defined in this class. Note `open` here, since
+            # it calls `_open`, but is actually in superclass
+            return lambda *args, **kw: getattr(type(self), item).__get__(self)(
+                *args, **kw
+            )
+        if item in ["__reduce_ex__"]:
+            raise AttributeError
+        if item in ["transaction"]:
+            # property
+            return type(self).transaction.__get__(self)
+        if item in ["_cache", "transaction_type"]:
+            # class attributes
+            return getattr(type(self), item)
+        if item == "__class__":
+            return type(self)
+        d = object.__getattribute__(self, "__dict__")
+        fs = d.get("fs", None)  # fs is not immediately defined
+        if item in d:
+            return d[item]
+        elif fs is not None:
+            if item in fs.__dict__:
+                # attribute of instance
+                return fs.__dict__[item]
+            # attributed belonging to the target filesystem
+            cls = type(fs)
+            m = getattr(cls, item)
+            if (inspect.isfunction(m) or inspect.isdatadescriptor(m)) and (
+                not hasattr(m, "__self__") or m.__self__ is None
+            ):
+                # instance method
+                return m.__get__(fs, cls)
+            return m  # class method or attribute
+        else:
+            # attributes of the superclass, while target is being set up
+            return super().__getattribute__(item)
 
 
 class GitLabFileSystem(AbstractFileSystem):
@@ -448,6 +555,145 @@ DirFileSystem.ls = dir_ls_p
 MonitoredSimpleCacheFileSystem.ls = mscf_ls_p
 
 
+def _ensure_string(path: str | Path | None) -> str:
+    if isinstance(path, Path):
+        return path.as_posix()
+    return str(path) if path is not None else ""
+
+
+def _normalize_path(path: str, sep: str = "/") -> str:
+    if not path:
+        return ""
+    if sep == "/":
+        path = path.replace("\\", "/")
+    normalized = posixpath.normpath(path)
+    if normalized == ".":
+        return ""
+    return normalized
+
+
+def _join_paths(base: str, rel: str, sep: str = "/") -> str:
+    base_norm = _normalize_path(base, sep)
+    rel = rel or ""
+    if not rel:
+        return base_norm
+    if rel.startswith(sep):
+        return _normalize_path(rel, sep)
+    if not base_norm:
+        return _normalize_path(rel, sep)
+    return _normalize_path(f"{base_norm.rstrip(sep)}{sep}{rel}", sep)
+
+
+def _is_within(base: str, target: str, sep: str = "/") -> bool:
+    base_norm = _normalize_path(base, sep)
+    target_norm = _normalize_path(target, sep)
+    if not base_norm:
+        return True
+    if base_norm == target_norm:
+        return True
+    prefix = f"{base_norm.rstrip(sep)}{sep}"
+    return target_norm.startswith(prefix)
+
+
+def _smart_join(base: str, rel: str, sep: str = "/") -> str:
+    """Join while avoiding duplicate overlapping segments.
+
+    Example:
+        base = 'ewn/mms2/stage1', rel = 'mms2/stage1/SC' -> 'ewn/mms2/stage1/SC'
+        base = 'ewn/mms2/stage1', rel = 'stage1/SC' -> 'ewn/mms2/stage1/SC'
+        base = 'ewn/mms2/stage1', rel = 'ewn/mms2/stage1/SC' -> 'ewn/mms2/stage1/SC'
+    """
+    base_norm = _normalize_path(base, sep)
+    rel_norm = _normalize_path(rel, sep)
+
+    if not rel_norm:
+        return base_norm
+    if not base_norm:
+        return rel_norm
+
+    # If rel already has the base prefix, accept as absolute
+    if _is_within(base_norm, rel_norm, sep):
+        return rel_norm
+
+    base_parts = [p for p in base_norm.split(sep) if p]
+    rel_parts = [p for p in rel_norm.split(sep) if p]
+
+    # Longest suffix of base matching a prefix of rel
+    max_overlap = 0
+    lim = min(len(base_parts), len(rel_parts))
+    for k in range(lim, 0, -1):
+        if base_parts[-k:] == rel_parts[:k]:
+            max_overlap = k
+            break
+
+    merged_parts = base_parts + rel_parts[max_overlap:]
+    # Preserve absolute root marker if base was absolute
+    leading = sep if base_norm.startswith(sep) else ""
+    return f"{leading}{sep.join(merged_parts)}"
+
+
+def _protocol_set(protocol: str | tuple[str, ...] | list[str]) -> set[str]:
+    if isinstance(protocol, (list, tuple, set)):
+        values = protocol
+    else:
+        values = [protocol]
+    normalized = set()
+    for value in values:
+        if value is None:
+            continue
+        value = value.lower()
+        if value in {"file", "local"}:
+            normalized.update({"file", "local"})
+        else:
+            normalized.add(value)
+    return normalized
+
+
+def _protocol_matches(requested: str, candidates: set[str]) -> bool:
+    if requested is None:
+        return True
+    requested = requested.lower()
+    if requested in {"file", "local"}:
+        return bool({"file", "local"} & candidates)
+    return requested in candidates
+
+
+def _strip_for_fs(fs: AbstractFileSystem, url: str) -> str:
+    if not url:
+        return ""
+    try:
+        return fs._strip_protocol(url)
+    except AttributeError:
+        _, path = split_protocol(url)
+        return path or ""
+
+
+def _detect_local_file_path(path: str) -> tuple[str, bool]:
+    """Return parent directory for local files that exist."""
+    if not path:
+        return "", False
+    try:
+        candidate = Path(path)
+    except (TypeError, ValueError):
+        return path, False
+    try:
+        if candidate.exists() and candidate.is_file():
+            parent = candidate.parent
+            parent_str = parent.as_posix()
+            return parent_str if parent_str != "." else "", True
+    except OSError:
+        # Non-existent or inaccessible paths fall back to original input.
+        pass
+    return path, False
+
+
+def _default_cache_storage(cache_path_hint: str | None) -> str:
+    base_cache_dir = Path.home() / ".fsspec_cache"
+    if cache_path_hint:
+        base_cache_dir = base_cache_dir / cache_path_hint
+    return base_cache_dir.as_posix()
+
+
 def filesystem(
     protocol_or_path: str | None = "",
     storage_options: Optional[Union[BaseStorageOptions, dict]] = None,
@@ -494,96 +740,179 @@ def filesystem(
     """
     if isinstance(protocol_or_path, Path):
         protocol_or_path = protocol_or_path.as_posix()
-    if not protocol_or_path:
-        # protocol_or_path = "file://"
-        base_path = ""
-        protocol = "file"
 
-    elif "://" in protocol_or_path:
-        base_path = protocol_or_path.split("://")[-1]
-        protocol = protocol_or_path.split("://")[0]
-    elif "/" in protocol_or_path or "." in protocol_or_path:
-        base_path = protocol_or_path
-        protocol = "file"
-    else:
-        protocol = protocol_or_path if protocol_or_path is not None else "file"
-        base_path = ""
+    raw_input = _ensure_string(protocol_or_path)
+    protocol_from_kwargs = kwargs.pop("protocol", None)
 
-    base_path = base_path or ""
-    normalized_base_path = base_path.rstrip("/\\")
+    provided_protocol: str | None = None
+    base_path_input: str = ""
 
-    if not normalized_base_path and base_path.startswith(("/", "\\")):
-        normalized_base_path = base_path[:1]
-
-    if normalized_base_path:
-        candidate = normalized_base_path
-        base_name = posixpath.basename(candidate)
-        _, extension = posixpath.splitext(base_name)
-
-        if extension:
-            base_path = posixpath.dirname(candidate)
+    if raw_input:
+        provided_protocol, remainder = split_protocol(raw_input)
+        if provided_protocol:
+            base_path_input = remainder or ""
         else:
-            base_path = candidate
+            base_path_input = remainder or raw_input
+            if base_fs is None and base_path_input in known_implementations:
+                provided_protocol = base_path_input
+                base_path_input = ""
     else:
-        base_path = normalized_base_path
+        base_path_input = ""
 
-    #print(f"Base path: {base_path}, Protocol: {protocol}")
+    base_path_input = base_path_input.replace("\\", "/")
+
+    if (
+        base_fs is None
+        and base_path_input
+        and (provided_protocol or protocol_from_kwargs) in {None, "file", "local"}
+    ):
+        detected_parent, is_file = _detect_local_file_path(base_path_input)
+        if is_file:
+            base_path_input = detected_parent
+
+    base_path = _normalize_path(base_path_input)
+    cache_path_hint = base_path
 
     if base_fs is not None:
-        if dirfs:
-            # base_path = protocol_or_path.split("://")[-1]
-            if base_fs.protocol == "dir":
-                if base_path != base_fs.path:
-                    fs = DirFileSystem(
-                        path=posixpath.join(
-                            base_fs.path,
-                            base_path.replace(base_fs.path, "").lstrip("/"),
-                        ),
-                        fs=base_fs.fs,
-                    )
+        if not dirfs:
+            raise ValueError("dirfs must be True when providing base_fs")
+
+        base_is_dir = isinstance(base_fs, DirFileSystem)
+        underlying_fs = base_fs.fs if base_is_dir else base_fs
+        underlying_protocols = _protocol_set(underlying_fs.protocol)
+        requested_protocol = provided_protocol or protocol_from_kwargs
+
+        if requested_protocol and not _protocol_matches(
+            requested_protocol, underlying_protocols
+        ):
+            raise ValueError(
+                f"Protocol '{requested_protocol}' does not match base filesystem protocol "
+                f"{sorted(underlying_protocols)}"
+            )
+
+        sep = getattr(underlying_fs, "sep", "/") or "/"
+        base_root = base_fs.path if base_is_dir else ""
+        base_root_norm = _normalize_path(base_root, sep)
+        cache_path_hint = base_root_norm
+
+        fs: AbstractFileSystem
+        path_for_cache = base_root_norm
+
+        if requested_protocol:
+            absolute_target = _strip_for_fs(underlying_fs, raw_input)
+            absolute_target = _normalize_path(absolute_target, sep)
+
+            if (
+                base_is_dir
+                and base_root_norm
+                and not _is_within(base_root_norm, absolute_target, sep)
+            ):
+                raise ValueError(
+                    f"Requested path '{absolute_target}' is outside the base directory "
+                    f"'{base_root_norm}'"
+                )
+
+            if base_is_dir and absolute_target == base_root_norm:
+                fs = base_fs
             else:
-                fs = DirFileSystem(path=base_path, fs=base_fs)
+                fs = DirFileSystem(path=absolute_target, fs=underlying_fs)
+
+            path_for_cache = absolute_target
+        else:
+            rel_input = base_path
+            if rel_input:
+                segments = [segment for segment in rel_input.split(sep) if segment]
+                if any(segment == ".." for segment in segments):
+                    raise ValueError(
+                        "Relative paths must not escape the base filesystem root"
+                    )
+
+                candidate = _normalize_path(rel_input, sep)
+                absolute_target = _smart_join(base_root_norm, candidate, sep)
+
+                if (
+                    base_is_dir
+                    and base_root_norm
+                    and not _is_within(base_root_norm, absolute_target, sep)
+                ):
+                    raise ValueError(
+                        f"Resolved path '{absolute_target}' is outside the base "
+                        f"directory '{base_root_norm}'"
+                    )
+
+                if base_is_dir and absolute_target == base_root_norm:
+                    fs = base_fs
+                else:
+                    fs = DirFileSystem(path=absolute_target, fs=underlying_fs)
+
+                path_for_cache = absolute_target
+            else:
+                fs = base_fs
+                path_for_cache = base_root_norm
+
+        cache_path_hint = path_for_cache
+
         if cached:
-            if fs.is_cache_fs:
+            if getattr(fs, "is_cache_fs", False):
                 return fs
-            fs = MonitoredSimpleCacheFileSystem(fs=fs, cache_storage=cache_storage)
+            storage = cache_storage
+            if storage is None:
+                storage = _default_cache_storage(cache_path_hint or None)
+            cached_fs = MonitoredSimpleCacheFileSystem(
+                fs=fs, cache_storage=storage, verbose=verbose
+            )
+            cached_fs.is_cache_fs = True
+            return cached_fs
 
-        return fs
-
-    protocol = (
-        protocol
-        or kwargs.get("protocol", None)
-        or (
-            storage_options.get("protocol", None)
-            if isinstance(storage_options, dict)
-            else getattr(storage_options, "protocol", None)
-        )
-    )
-
-    if protocol == "file" or protocol == "local":
-        fs = fsspec_filesystem(protocol)
-        fs.is_cache_fs = False
-        if dirfs:
-            fs = DirFileSystem(path=base_path or Path.cwd(), fs=fs)
+        if not hasattr(fs, "is_cache_fs"):
             fs.is_cache_fs = False
         return fs
 
-    if isinstance(storage_options, dict):
-        storage_options = storage_options_from_dict(protocol, storage_options)
+    protocol = provided_protocol or protocol_from_kwargs
+    if protocol is None:
+        if isinstance(storage_options, dict):
+            protocol = storage_options.get("protocol")
+        else:
+            protocol = getattr(storage_options, "protocol", None)
 
-    if storage_options is None:
-        storage_options = storage_options_from_dict(protocol, kwargs)
+    protocol = protocol or "file"
+    protocol = protocol.lower()
+
+    if protocol in {"file", "local"}:
+        fs = fsspec_filesystem(protocol)
+        if dirfs:
+            dir_path: str | Path = base_path or Path.cwd()
+            fs = DirFileSystem(path=dir_path, fs=fs)
+            cache_path_hint = _ensure_string(dir_path)
+        if not hasattr(fs, "is_cache_fs"):
+            fs.is_cache_fs = False
+    else:
+        storage_opts = storage_options
+        if isinstance(storage_opts, dict):
+            storage_opts = storage_options_from_dict(protocol, storage_opts)
+        if storage_opts is None:
+            storage_opts = storage_options_from_dict(protocol, kwargs)
+        fs = storage_opts.to_filesystem()
+        if dirfs and base_path:
+            fs = DirFileSystem(path=base_path, fs=fs)
+            cache_path_hint = base_path
+        if not hasattr(fs, "is_cache_fs"):
+            fs.is_cache_fs = False
 
-    fs = storage_options.to_filesystem()
-    fs.is_cache_fs = False
-    if dirfs and len(base_path):
-        fs = DirFileSystem(path=base_path, fs=fs)
-        fs.is_cache_fs = False
     if cached:
-        if cache_storage is None:
-            cache_storage = (Path.cwd() / base_path).as_posix()
-        fs = MonitoredSimpleCacheFileSystem(fs=fs, cache_storage=cache_storage)
-        fs.is_cache_fs = True
+        if getattr(fs, "is_cache_fs", False):
+            return fs
+        storage = cache_storage
+        if storage is None:
+            storage = _default_cache_storage(cache_path_hint or None)
+        cached_fs = MonitoredSimpleCacheFileSystem(
+            fs=fs, cache_storage=storage, verbose=verbose
+        )
+        cached_fs.is_cache_fs = True
+        return cached_fs
+
+    if not hasattr(fs, "is_cache_fs"):
+        fs.is_cache_fs = False
 
     return fs
 
