@@ -5,16 +5,18 @@ import uuid
 from typing import Any, Generator
 
 import orjson
+
 # import polars as pl
 import pyarrow as pa
 import pyarrow.dataset as pds
 import pyarrow.parquet as pq
 from fsspec import AbstractFileSystem
-#from pydala.dataset import ParquetDataset
+# from pydala.dataset import ParquetDataset
 
 from ..utils.misc import path_to_glob, run_parallel
 from ..utils.polars import opt_dtype as opt_dtype_pl
 from ..utils.polars import pl
+
 # from ..utils.polars import unify_schemas as unfify_schemas_pl
 from ..utils.pyarrow import cast_schema, convert_large_types_to_normal
 from ..utils.pyarrow import opt_dtype as opt_dtype_pa
@@ -1483,7 +1485,7 @@ def pyarrow_parquet_dataset(
     """
     if not self.isfile(path):
         path = posixpath.join(path, "_metadata")
-    return pds.dataset(
+    return pds.parquet_dataset(
         path,
         filesystem=self,
         partitioning=partitioning,
@@ -2122,6 +2124,17 @@ def write_pyarrow_dataset(
 #     )
 
 
+def clear_cache(fs: AbstractFileSystem | None):
+    if hasattr(fs, "dircache"):
+        if hasattr(fs, "fs"):
+            fs.fs.invalidate_cache()
+            fs.fs.clear_instance_cache()
+        else:
+            fs.invalidate_cache()
+            fs.clear_instance_cache()
+
+
+AbstractFileSystem.clear_cache = clear_cache
 AbstractFileSystem.read_json_file = read_json_file
 AbstractFileSystem.read_json = read_json
 AbstractFileSystem.read_csv_file = read_csv_file
@@ -2130,7 +2143,7 @@ AbstractFileSystem.read_parquet_file = read_parquet_file
 AbstractFileSystem.read_parquet = read_parquet
 AbstractFileSystem.read_files = read_files
 AbstractFileSystem.pyarrow_dataset = pyarrow_dataset
-#AbstractFileSystem.pydala_dataset = pydala_dataset
+# AbstractFileSystem.pydala_dataset = pydala_dataset
 AbstractFileSystem.pyarrow_parquet_dataset = pyarrow_parquet_dataset
 AbstractFileSystem.write_parquet = write_parquet
 AbstractFileSystem.write_json = write_json
@@ -2138,4 +2151,4 @@ AbstractFileSystem.write_csv = write_csv
 AbstractFileSystem.write_file = write_file
 AbstractFileSystem.write_files = write_files
 AbstractFileSystem.write_pyarrow_dataset = write_pyarrow_dataset
-#AbstractFileSystem.write_pydala_dataset = write_pydala_dataset
+# AbstractFileSystem.write_pydala_dataset = write_pydala_dataset
