@@ -2,9 +2,15 @@
 
 from .base import BaseStorageOptions
 from .cloud import AwsStorageOptions, AzureStorageOptions, GcsStorageOptions
-from .core import (LocalStorageOptions, StorageOptions, from_dict, from_env,
-                   infer_protocol_from_uri, merge_storage_options,
-                   storage_options_from_uri)
+from .core import (
+    LocalStorageOptions,
+    StorageOptions,
+    from_dict,
+    from_env,
+    infer_protocol_from_uri,
+    merge_storage_options,
+    storage_options_from_uri,
+)
 from .git import GitHubStorageOptions, GitLabStorageOptions
 
 __all__ = [
